/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.app.gui;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Desktop;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import picto.app.interfaces.Localizable;
import picto.utils.Config;
import picto.utils.Text;

import static picto.utils.Text._;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class AboutWindow extends JFrame implements Localizable, ActionListener{
    private static final long serialVersionUID = 7879846356384791L;

    /** Creates new form AboutWindow */
    public AboutWindow() {
        initComponents();
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        tabbedPane.setFocusable(false);
        licenseTextArea.setEditable(false);
        changeLogEditorPane.setEditable(false);
        creditsTextArea.setEditable(false);
        creditsTextArea.setCaretPosition(0);
        homepageButton.setFocusable(false);
        changeLogEditorPane.setText(loadChangeLog());
        changeLogEditorPane.setCaretPosition(0);
        licenseTextArea.setText(loadLicense());
        licenseTextArea.setCaretPosition(0);
        updateTexts();
        addWeakReference();
    }

    @Override
    public final void updateTexts() {
        setTitle(_("About..."));
        tabbedPane.setTitleAt(0, _("Presentation"));
        tabbedPane.setTitleAt(1, _("License"));
        tabbedPane.setTitleAt(2, _("Credits"));
        tabbedPane.setTitleAt(3, _("ChangeLog"));
        titleLabel.setText(Config.getAppName());
        versionLabel.setText(_("Version")+" " + Config.getAppVersion());
        descriptionLabel.setText(_(Config.getAppDescription()));
        homepageButton.setText(_("PictoGame Homepage"));
        homepageButton.setToolTipText(Config.getAppURL());
        creditsTextArea.setText(loadCredits());
        creditsTextArea.setCaretPosition(0);
    }

    @Override
    public final void addWeakReference() {
        Text.addWeakReference(this);
    }

    public static String loadCredits(){
        String credits = "";
        String temp;
        try {
            BufferedReader reader = new BufferedReader(new InputStreamReader(
                            AboutWindow.class.getResourceAsStream("/AUTHORS")));
            do{
                temp = reader.readLine();
                if (temp == null){
                } else if(temp.contains("Developers:")){
                    credits += "  "+_("Developers:")+"\n";
                } else if (temp.contains("Translators:")){
                    credits += "  "+_("Translators:")+"\n";
                } else if(temp.contains("Bug reporters:")){
                    credits += "  "+_("Bug reporters:")+"\n";
                } else if(temp.contains("Special thanks to:")){
                    credits += "  "+_("Special thanks to:")+"\n";
                } else{
                    credits += temp +"\n";
                }
            } while(temp != null);
            reader.close();
        } catch (FileNotFoundException ex) {
            Logger.getLogger(AboutWindow.class.getName())
                    .log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(AboutWindow.class.getName())
                    .log(Level.SEVERE, null, ex);
        }
        return credits;
    }

    public static String loadChangeLog(){
        String changeLog = "";
        String temp;
        try {
            BufferedReader reader = new BufferedReader(new InputStreamReader(
                        AboutWindow.class.getResourceAsStream("/ChangeLog")));
            temp = reader.readLine();
            while(temp != null){
                changeLog += temp +"\n";
                temp = reader.readLine();
            }
            reader.close();
        } catch (FileNotFoundException ex) {
            Logger.getLogger(AboutWindow.class.getName())
                    .log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(AboutWindow.class.getName())
                    .log(Level.SEVERE, null, ex);
        }
        return changeLog;
    }

    private static final int GPL_LENGHT = 0x7E01;
    private static String loadLicense(){
        StringBuilder _GPL = new StringBuilder(GPL_LENGHT);
        String temp;
        try {
            BufferedReader reader = new BufferedReader(new InputStreamReader(
                          AboutWindow.class.getResourceAsStream("/COPYING")));
            temp = reader.readLine();
            while(temp != null){
                _GPL.append(temp).append("\n");
                temp = reader.readLine();
            } 
            reader.close();
        } catch (FileNotFoundException ex) {
            Logger.getLogger(AboutWindow.class.getName())
                    .log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(AboutWindow.class.getName())
                    .log(Level.SEVERE, null, ex);
        }
        if (_GPL.length() == GPL_LENGHT){
            return _GPL.toString();
        } else{
            return _("Unable to load the License file... Sorry");
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        GridBagConstraints gridBagConstraints;

        tabbedPane = new JTabbedPane();
        presentationPanel = new JPanel();
        titleLabel = new JLabel();
        versionLabel = new JLabel();
        descriptionLabel = new JLabel();
        homepageButton = new JButton();
        licensePanel = new JPanel();
        licenseScrollPane = new JScrollPane();
        licenseTextArea = new JTextPane();
        creditsPanel = new JPanel();
        jScrollPane2 = new JScrollPane();
        creditsTextArea = new JTextPane();
        ChangeLog = new JPanel();
        jScrollPane1 = new JScrollPane();
        changeLogEditorPane = new JTextPane();

        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

        tabbedPane.setTabPlacement(JTabbedPane.BOTTOM);
        tabbedPane.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        tabbedPane.setDoubleBuffered(true);

        presentationPanel.setLayout(new GridBagLayout());

        titleLabel.setFont(new Font("URW Chancery L", 1, 48));
        titleLabel.setForeground(new Color(77, 81, 126));
        titleLabel.setHorizontalAlignment(SwingConstants.CENTER);
        titleLabel.setText("Picto");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 0, 0, 0);
        presentationPanel.add(titleLabel, gridBagConstraints);

        versionLabel.setFont(new Font("DejaVu Sans", 0, 18));
        versionLabel.setHorizontalAlignment(SwingConstants.CENTER);
        versionLabel.setText("Version x.x.x");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 0, 4, 0);
        presentationPanel.add(versionLabel, gridBagConstraints);

        descriptionLabel.setFont(new Font("DejaVu Sans", 1, 13));
        descriptionLabel.setForeground(new Color(215, 151, 60));
        descriptionLabel.setHorizontalAlignment(SwingConstants.CENTER);
        descriptionLabel.setText("Description");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(3, 0, 3, 0);
        presentationPanel.add(descriptionLabel, gridBagConstraints);

        homepageButton.setText("HomePage");
        homepageButton.setBorderPainted(false);
        homepageButton.setFocusable(false);
        homepageButton.addActionListener(this);
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        presentationPanel.add(homepageButton, gridBagConstraints);

        tabbedPane.addTab("Presentation", presentationPanel);

        licensePanel.setLayout(new GridLayout(1, 0));

        licenseScrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        licenseTextArea.setFont(new Font("Courier New", 0, 13));
        licenseScrollPane.setViewportView(licenseTextArea);

        licensePanel.add(licenseScrollPane);

        tabbedPane.addTab("License", licensePanel);

        creditsPanel.setLayout(new GridLayout(1, 1));

        creditsTextArea.setFont(new Font("Courier New", 0, 13));
        jScrollPane2.setViewportView(creditsTextArea);

        creditsPanel.add(jScrollPane2);

        tabbedPane.addTab("Credits", creditsPanel);

        changeLogEditorPane.setFont(new Font("Courier New", 0, 13));
        jScrollPane1.setViewportView(changeLogEditorPane);

        GroupLayout ChangeLogLayout = new GroupLayout(ChangeLog);
        ChangeLog.setLayout(ChangeLogLayout);
        ChangeLogLayout.setHorizontalGroup(
            ChangeLogLayout.createParallelGroup(Alignment.LEADING)
            .addComponent(jScrollPane1, GroupLayout.DEFAULT_SIZE, 578, Short.MAX_VALUE)
        );
        ChangeLogLayout.setVerticalGroup(
            ChangeLogLayout.createParallelGroup(Alignment.LEADING)
            .addComponent(jScrollPane1, GroupLayout.DEFAULT_SIZE, 232, Short.MAX_VALUE)
        );

        tabbedPane.addTab("ChangeLog", ChangeLog);

        GroupLayout layout = new GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(Alignment.LEADING)
            .addComponent(tabbedPane, GroupLayout.DEFAULT_SIZE, 586, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(Alignment.LEADING)
            .addComponent(tabbedPane, GroupLayout.PREFERRED_SIZE, 270, Short.MAX_VALUE)
        );

        pack();
    }

    // Code for dispatching events from components to event handlers.

    public void actionPerformed(java.awt.event.ActionEvent evt) {
        if (evt.getSource() == homepageButton) {
            AboutWindow.this.homepageButtonActionPerformed(evt);
        }
    }// </editor-fold>//GEN-END:initComponents

    private void homepageButtonActionPerformed(ActionEvent evt) {//GEN-FIRST:event_homepageButtonActionPerformed
        if (Desktop.isDesktopSupported()) {
            Desktop desktop = Desktop.getDesktop();
            try {
                desktop.browse(new URI(Config.getAppURL()));
            } catch (URISyntaxException ex) {
                Logger.getLogger(AboutWindow.class.getName())
                        .log(Level.SEVERE, null, ex);
            } catch (IOException ex) {
                Logger.getLogger(AboutWindow.class.getName())
                        .log(Level.SEVERE, null, ex);
            }
        }
}//GEN-LAST:event_homepageButtonActionPerformed
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private JPanel ChangeLog;
    private JTextPane changeLogEditorPane;
    private JPanel creditsPanel;
    private JTextPane creditsTextArea;
    private JLabel descriptionLabel;
    private JButton homepageButton;
    private JScrollPane jScrollPane1;
    private JScrollPane jScrollPane2;
    private JPanel licensePanel;
    private JScrollPane licenseScrollPane;
    private JTextPane licenseTextArea;
    private JPanel presentationPanel;
    private JTabbedPane tabbedPane;
    private JLabel titleLabel;
    private JLabel versionLabel;
    // End of variables declaration//GEN-END:variables

}
