/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.app.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractListModel;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import picto.utils.Picto;
import picto.app.interfaces.Localizable;
import picto.utils.Text;

import static picto.utils.Text._;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class GameLoader extends JDialog implements Localizable, KeyListener{
    private static final long serialVersionUID = -63765627876372891L;
    private MainWindow father;
    private int NUMBER_OF_PATTERNS;

    /** 
     * Creates new form GameLoader
     * @param father 
     */
    public GameLoader(MainWindow father) {
        this.father = father;
        setModal(true);
        initComponents();
        descriptionTextPane.setEditable(false);
        descriptionTextPane.setContentType("text/html");
        setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
        addActionListeners();
        loadList();
        updateTexts();
        addWeakReference();
        updateList(generateDisplayedGameList());
        rowSpinner.setModel(new SpinnerNumberModel(10, 5, 64, 1));
        columnSpinner.setModel(new SpinnerNumberModel(10, 5, 64, 1));
        // Yes... I know this is stupid... but windows 7 needs it...
        setSize(getSize().width+1, getSize().height);
    }

    public void getRandomGame(){
        father.setGame(games[new Random().nextInt(games.length)]);
    }

    @Override 
    public final void updateTexts() {
        setTitle(_("Load game"));
        authorsComboBox.setModel(new DefaultComboBoxModel<String>(getAuthorNameList()));
        conditionComboBox.setModel(new DefaultComboBoxModel<String>(new String[]
          {_("Any size"), _("Bigger than"), _("Smaller than"), _("Equal to")}));
        authorsComboBox.setToolTipText(_("Author"));
        conditionComboBox.setToolTipText(_("Size condition"));
        filterPanel.setBorder(BorderFactory.createTitledBorder(_("Filter")));
        rowsLabel.setText(_("Rows"));
        columnsLabel.setText(_("Columns"));
        openFromFileButton.setText(_("Open from file"));
        openSelectedButton.setText(_("Open"));
        rowSpinner.setEnabled(conditionComboBox.getSelectedIndex() != 0);
        columnSpinner.setEnabled(conditionComboBox.getSelectedIndex() != 0);
        setDescription();
    }

    @Override
    public final void addWeakReference() {
        Text.addWeakReference(this);
    }

    /**
     * Sets the game description in the description panel
     */
    private void setDescription(){
        int idx = gameList.getSelectedIndex();
        
        if (idx == -1){
            descriptionTextPane.setText(_("No element selected"));
            return;
        }
        
        StringBuilder tmpStr = new StringBuilder(200);
        tmpStr.append("<html><b>")
              .append(_("Name:"))
              .append(_b)
              .append(displayedGames[idx].getTitle())
              .append(br)
              .append(_("Author:"))
              .append(_b)
              .append(_(displayedGames[idx].getAuthor()))
              .append(br)
              .append(_("Date:"))
              .append(_b)
              .append(displayedGames[idx].getDate())
              .append(br)
              .append(_("Size:"))
              .append(_b)
              .append(displayedGames[idx].getNumberOfRows())
              .append("x")
              .append(displayedGames[idx].getNumberOfColumns())
              .append("</html>");
        descriptionTextPane.setText(tmpStr.toString());
    }
    
    private static final String br = "<br><b>";
    private static final String _b = " </b>";


    /** Internal path to the patterns*/
    private static final String path = "/res/patterns/";
    /** This array contains the games that are currently shown in the list*/
    private static Picto[] displayedGames;
    /** This array contains all the games*/
    private static Picto[] games;

    /**
     * This method loads all the games in the array
     */
    private void loadList(){
        BufferedReader reader = new BufferedReader(new InputStreamReader(
                 GameLoader.class.getResourceAsStream(path + "patterns.list")));
        ArrayList<String> list = new ArrayList<String>(200);
        //Load the 'patterns.list' file
        try {
            String line = reader.readLine();
            while (line != null){
                line = line.trim();
                if(!line.startsWith("#") && !line.isEmpty()){
                    list.add(line);
                    line = reader.readLine();
                    NUMBER_OF_PATTERNS++;
                }
            }
            reader.close();
        } catch (IOException ex) {
            Logger.getLogger(GameLoader.class.getName()).log(Level.SEVERE, null, ex);
        }
        // Load the games that appear in 'patterns.list'
        games = new Picto[list.size()];
        int i = 0;
        for (String element : list){
            games[i++] = new Picto(GameLoader.class.getResourceAsStream
                                  (path + element + ".pattern"), true);
        }
    }

    /**
     * Generates a string array containing the names of all the authors
     *
     * @return String array with authors names
     */
    private String[] getAuthorNameList(){
        ArrayList<String> authorListName = new ArrayList<String>(10);
        authorListName.add(_("All..."));
        for (Picto game : games){
            String name = game.getAuthor();
            if (name.isEmpty()){
                name = _("Unknown");
            }
            name = "<html>" + name + "</html>";//Sorry for this...
            if (!authorListName.contains(name)){
                authorListName.add(name);
            }
        }
        return authorListName.toArray(new String[0]);
    }

    /**
     * Add the action listeners to all the elements
     */
    private void addActionListeners(){
        openFromFileButton.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                JFileChooser fc = new JFileChooser();
                fc.setDialogTitle(_("Open pattern"));
                fc.setDialogType(JFileChooser.OPEN_DIALOG);
                fc.setMultiSelectionEnabled(false);
                fc.setFileHidingEnabled(true);
                fc.setFileFilter(new FileNameExtensionFilter(
                                        _("PictoGame Pattern file"), "pattern", "sgriddler"));
                if (fc.showOpenDialog(father) == JFileChooser.APPROVE_OPTION){
                    try{
                        if (fc.getSelectedFile().getCanonicalFile().toString().endsWith("pattern")){
                            Picto temp = new Picto(new BufferedReader(
                                            new FileReader(fc.getSelectedFile())));
                            father.setGame(temp);
                            setVisible(false);
                        } else if (fc.getSelectedFile().getCanonicalFile().toString().endsWith("sgriddler")){
                            father.setGame(Picto.loadBoardFromSGriddlerFile(fc.getSelectedFile()));
                            setVisible(false);
                        }
                    } catch (Exception ex){
                        JOptionPane.showMessageDialog(null, _("Error opening file!"),
                                        _("Oops..."), JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        });
        authorsComboBox.addItemListener(new java.awt.event.ItemListener() {
            @Override public void itemStateChanged(java.awt.event.ItemEvent evt) {
                filterBySize();
            }
        });
        conditionComboBox.addItemListener(new java.awt.event.ItemListener() {
            @Override public void itemStateChanged(java.awt.event.ItemEvent evt) {
                rowSpinner.setEnabled(conditionComboBox.getSelectedIndex() != 0);
                columnSpinner.setEnabled(conditionComboBox.getSelectedIndex() != 0);
                filterBySize();
            }
        });
        openSelectedButton.addActionListener(new ActionListener() {
            @Override public void actionPerformed(ActionEvent e) {
                father.setGame(displayedGames[gameList.getSelectedIndex()]);
                setVisible(false);
            }
        });
        gameList.addListSelectionListener(new ListSelectionListener() {
            @Override public void valueChanged(ListSelectionEvent evt) {
                setDescription();
            }
        });
        gameList.addMouseListener(new MouseAdapter() {
            @Override public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2){
                    openSelectedButton.doClick();
                }
            }
        });
        rowSpinner.addMouseWheelListener(new MouseWheelListener() {
            @Override public void mouseWheelMoved(MouseWheelEvent e) {
                if (e.getWheelRotation() < 0){
                    if (rowSpinner.getNextValue() != null) {
                        rowSpinner.setValue(rowSpinner.getNextValue());
                    }
                } else {
                    if (rowSpinner.getPreviousValue() != null) {
                        rowSpinner.setValue(rowSpinner.getPreviousValue());
                    } 
                }
            }
        });
        columnSpinner.addMouseWheelListener(new MouseWheelListener() {
            @Override public void mouseWheelMoved(MouseWheelEvent e) {
                if (e.getWheelRotation() < 0){
                    if (columnSpinner.getNextValue() != null) {
                        columnSpinner.setValue(columnSpinner.getNextValue());
                    } 
                } else {
                    if (columnSpinner.getPreviousValue() != null) {
                        columnSpinner.setValue(columnSpinner.getPreviousValue());
                    } 
                }
            }
        });
        rowSpinner.addChangeListener(new ChangeListener() {
            @Override public void stateChanged(ChangeEvent evt) {
                filterBySize();
            }
        });
        columnSpinner.addChangeListener(new ChangeListener() {
            @Override public void stateChanged(ChangeEvent evt) {
                filterBySize();
            }
        });
    }

    /**
     * Generates the game list by author
     *
     * @return Game list by selected author
     */
    private ArrayList<Picto> generateDisplayedGameList(){
        String name = (String)authorsComboBox.getSelectedItem();
        ArrayList<Picto> list = new ArrayList<Picto>(NUMBER_OF_PATTERNS);
        for (Picto game : games){
            String foo = "<html>"+game.getAuthor()+"</html>";//Sorry...
            if (foo.equals("<html></html>")){
                foo = _("Unknown");
            }
            if (authorsComboBox.getSelectedIndex() == 0){
                list.add(game);
            } else if (foo.startsWith(name)){
                list.add(game);
            }
        }
        return list;
    }

    /**
     * Filters the list generated by generateDisplayedGameList()
     */
    private void filterBySize(){
        int rows = (Integer)rowSpinner.getValue();
        int columns = (Integer)columnSpinner.getValue();
        ArrayList<Picto> list = generateDisplayedGameList();
        ArrayList<Picto> lis  = new ArrayList<Picto>(NUMBER_OF_PATTERNS);
        switch (conditionComboBox.getSelectedIndex()){
            case 0: // Any size
                lis = list;
                break;
            case 1: // Bigger than
                for (Picto game : list){
                    if (game.getNumberOfColumns() > columns
                     && game.getNumberOfRows   () > rows){
                        lis.add(game);
                    }
                }
                break;
            case 2: // Smaller than
                for (Picto game : list){
                    if (game.getNumberOfColumns() < columns
                     && game.getNumberOfRows   () < rows){
                        lis.add(game);
                    }
                }
                break;
            case 3: // Equal to
                for (Picto game : list){
                    if (game.getNumberOfColumns() == columns
                     && game.getNumberOfRows   () == rows){
                        lis.add(game);
                    }
                }
                break;
            default: // WTF!?
                assert false : "Filter list found unknown operator";
                break;
        }
        updateList(filterByContent(lis, filterTextField.getText().toLowerCase()));
    }

    /**
     * Generates the list of names, and adds the elements to the panel
     *
     * @param list filtered list by author and size
     */
    private void updateList(ArrayList<Picto> list){
        int displayedGamesLength = list.size();
        int length = String.valueOf(displayedGamesLength).length()
                ;
        displayedGames = list.toArray(new Picto[0]);
        openSelectedButton.setEnabled(!list.isEmpty());
        final String[] elements = new String[displayedGamesLength];
        
        for (int i = 0; i < displayedGamesLength; i++){
            builder.append(zeros(i, length))
                   .append(i+1)
                   .append('.').append(' ')
                   .append(displayedGames[i].getTitle());
            elements[i] = builder.toString();
            builder.delete(0, builder.length());
        }
        
        gameList.setModel(new AbstractListModel<String>() {
            private static final long serialVersionUID = 19415617846756343L;
            String[] strings = elements;
            @Override public int getSize()              {return strings.length;}
            @Override public String getElementAt(int i) {return strings[i];}
        });
        
        gameList.setSelectedIndex(0);
    }
    
    private StringBuilder builder = new StringBuilder(60);
    private char[][] zeros = new char[][]{
        {},
        {'0'},
        {'0','0'},
        {'0','0','0'},
        {'0','0','0','0'},
        {'0','0','0','0','0'},
    };
    private char[] zeros(int i, int lenght){
        int n_zeros = lenght - String.valueOf(i).length();
        
        if (n_zeros < zeros.length){
            return zeros[n_zeros];
        }
        
        char[] temp = new char[n_zeros];
        Arrays.fill(temp, '0')
                ;
        return temp;
    }

    private ArrayList<Picto> filterByContent (ArrayList<Picto> list, String filter){
        if (filter.isEmpty()){
            return list;
        }
        
        ArrayList<Picto> temp = new ArrayList<Picto>(list.size());
        for (Picto pic : list){
            if (pic.getTitle().toLowerCase().indexOf(filter) != -1){
                temp.add(pic);
            }
        }
        
        return temp;
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        gameList = new javax.swing.JList<String>();
        openSelectedButton = new javax.swing.JButton();
        openFromFileButton = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        descriptionTextPane = new javax.swing.JTextPane();
        filterPanel = new javax.swing.JPanel();
        authorsComboBox = new javax.swing.JComboBox<String>();
        columnSpinner = new javax.swing.JSpinner();
        rowSpinner = new javax.swing.JSpinner();
        columnsLabel = new javax.swing.JLabel();
        rowsLabel = new javax.swing.JLabel();
        conditionComboBox = new javax.swing.JComboBox<String>();
        filterTextField = new javax.swing.JTextField();

        gameList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jScrollPane1.setViewportView(gameList);

        openSelectedButton.setText("Open");

        openFromFileButton.setText("Open from file");

        descriptionTextPane.setFont(new java.awt.Font("DejaVu Sans", 0, 14)); // NOI18N
        jScrollPane2.setViewportView(descriptionTextPane);

        filterPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Filter"));

        rowSpinner.setModel(new javax.swing.SpinnerNumberModel(10, 5, 64, 1));

        columnsLabel.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
        columnsLabel.setText("Columns:");

        rowsLabel.setHorizontalAlignment(javax.swing.SwingConstants.TRAILING);
        rowsLabel.setText("Rows:");

        javax.swing.GroupLayout filterPanelLayout = new javax.swing.GroupLayout(filterPanel);
        filterPanel.setLayout(filterPanelLayout);
        filterPanelLayout.setHorizontalGroup(
            filterPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(filterPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(filterPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(filterPanelLayout.createSequentialGroup()
                        .addGroup(filterPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(rowsLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 152, Short.MAX_VALUE)
                            .addComponent(columnsLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 152, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(filterPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(rowSpinner, javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(columnSpinner, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 79, Short.MAX_VALUE)))
                    .addComponent(authorsComboBox, javax.swing.GroupLayout.Alignment.TRAILING, 0, 243, Short.MAX_VALUE)
                    .addComponent(conditionComboBox, 0, 243, Short.MAX_VALUE))
                .addContainerGap())
        );
        filterPanelLayout.setVerticalGroup(
            filterPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, filterPanelLayout.createSequentialGroup()
                .addComponent(authorsComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(conditionComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(filterPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(columnSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(columnsLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(filterPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(rowSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(rowsLabel))
                .addContainerGap())
        );

        filterTextField.addKeyListener(this);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(filterTextField, javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 245, Short.MAX_VALUE))
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 279, Short.MAX_VALUE)
                            .addComponent(filterPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addGap(68, 68, 68)
                        .addComponent(openFromFileButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(openSelectedButton)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 372, Short.MAX_VALUE)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 203, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(filterPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(openSelectedButton)
                    .addComponent(openFromFileButton)
                    .addComponent(filterTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        pack();
    }

    // Code for dispatching events from components to event handlers.

    public void keyPressed(KeyEvent evt) {
    }

    public void keyReleased(KeyEvent evt) {
        if (evt.getSource() == filterTextField) {
            GameLoader.this.filterTextFieldKeyReleased(evt);
        }
    }

    public void keyTyped(KeyEvent evt) {
    }// </editor-fold>//GEN-END:initComponents

    private void filterTextFieldKeyReleased(KeyEvent evt) {//GEN-FIRST:event_filterTextFieldKeyReleased
        filterBySize();
    }//GEN-LAST:event_filterTextFieldKeyReleased

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private JComboBox<String> authorsComboBox;
    private JSpinner columnSpinner;
    private JLabel columnsLabel;
    private JComboBox<String> conditionComboBox;
    private JTextPane descriptionTextPane;
    private JPanel filterPanel;
    private JTextField filterTextField;
    private JList<String> gameList;
    private JScrollPane jScrollPane1;
    private JScrollPane jScrollPane2;
    private JButton openFromFileButton;
    private JButton openSelectedButton;
    private JSpinner rowSpinner;
    private JLabel rowsLabel;
    // End of variables declaration//GEN-END:variables

}
