/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received generateList copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.app.gui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import javax.swing.JPanel;
import picto.utils.Config;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class PreviewPanel extends JPanel implements ComponentListener{
    private static final long serialVersionUID = 89680767896875541L;
    private boolean[][] drawing;
    private int rows, columns;

    public PreviewPanel(int number_of_rows, int number_of_columns){
        drawing = new boolean[number_of_rows][number_of_columns];
        columns = number_of_columns;
        rows = number_of_rows;
        addComponentListener(this);
    }

    /**
     * Sets the new size of the pattern.<br>
     * This is used every time the pattern changes
     *
     * @param number_of_rows
     * @param number_of_columns
     */
    public void setDrawingSize(int number_of_rows, int number_of_columns){
        color = Config.getColor("colorButtonBackgroundSomething");
        drawing = new boolean[number_of_rows][number_of_columns];
        columns = number_of_columns;
        rows = number_of_rows;
        repaint();
    }

    public void setColor(Color col){
        color = col;
        repaint();
    }

    /**
     * Sets the state of a new 'pixel'
     *
     * @param row 
     * @param column 
     * @param painted <code>true</code> if it should be painted and <code>false
     * </code> if it shouldn't
     */
    public void setElement(int row, int column, boolean painted){
        // Act if and only if the cell status changes
        if (drawing[row][column] != painted){
            drawing[row][column] = painted;
            repaint();
        }
    }

    private Color color;
    private int W_HEIGHT, W_WIDTH;
    private boolean W_VISIBLE;
    @Override 
    public void paintComponent(Graphics g){
        // Don't hate me, I will improve this over time, I just want it working
        // for now!
        // @FIXME The entire panel is draw every time you make a click...
        if (W_VISIBLE && columns != 0){
            g.clearRect(0, 0, W_WIDTH, W_HEIGHT);
            Graphics2D gg = (Graphics2D)g;
            int size = Math.min(W_HEIGHT / rows, W_WIDTH / columns);
            int sXcol = size * columns, sXrow = size * rows;
            int xOffset = (W_WIDTH  - sXcol) / 2;
            int yOffset = (W_HEIGHT - sXrow) / 2;
            gg.setColor(Color.WHITE);
            gg.fillRect(xOffset, yOffset, sXcol, sXrow);
            gg.setColor(color);
            gg.drawRect(xOffset, yOffset, sXcol, sXrow);
            
            for (int i = 0; i < rows; i++){
                for (int j = 0; j < columns; j++){
                    if (drawing[i][j]){
                        gg.fillRect(xOffset+j*size, yOffset+i*size, size, size);
                    }
                }
            }
        } 
    }

    @Override
    public void setVisible(boolean stat){
        super.setVisible(stat);
        W_VISIBLE = stat;
    }
    
    @Override
    public void componentResized(ComponentEvent e) {
        W_HEIGHT = getHeight();
        W_WIDTH = getWidth();
        repaint();
    }

    @Override public void componentMoved(ComponentEvent e) {}
    @Override public void componentShown(ComponentEvent e) {}
    @Override public void componentHidden(ComponentEvent e) {}

}
