/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.utils;

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class Config {
    private static Properties properties;
    private static Properties defProperties;

    static {
        dktpath = System.getProperty("user.home")+"/.dkt";
        pictopath = "/pictogame";
        loadDefaults();
        loadConfig();
    }

    private static final String COLOR_ = "color.";
    private static final String FONT_ = "font.";
    private static final String _NAME = ".name";
    private static final String _STYLE = ".style";
    private static final String _SIZE = ".size";
    public static Color getDefaultColor(String key){
        return new Color(Integer.valueOf(defProperties.getProperty(COLOR_ + key)));
    }

    public static Color getColor(String key){
        return new Color(Integer.valueOf(properties.getProperty(COLOR_ + key)));
    }

    public static void setColor(String key, Color color){
        properties.setProperty(COLOR_ + key, String.valueOf(color.getRGB()));
    }
    
    public static Font getDefaultFont(String key){
        String name = defProperties.getProperty(FONT_ + key + _NAME);
        int style = Integer.valueOf(defProperties.getProperty(FONT_ + key + _STYLE));
        int size = Integer.valueOf(defProperties.getProperty(FONT_ + key + _SIZE));
        return new Font(name, style, size);
    }

    public static Font getFont(String key){
        String name = properties.getProperty(FONT_ + key + _NAME);
        int style = Integer.valueOf(properties.getProperty(FONT_ + key + _STYLE));
        int size = Integer.valueOf(properties.getProperty(FONT_ + key + _SIZE));
        return new Font(name, style, size);
    }

    public static void setFont(String key, Font font){
        properties.setProperty(FONT_ + key + _NAME, font.getFontName());
        properties.setProperty(FONT_ + key + _STYLE, String.valueOf(font.getStyle()));
        properties.setProperty(FONT_ + key + _SIZE, String.valueOf(font.getSize()));
    }

    public static int getLocale(){
        return Integer.valueOf(properties.getProperty("locale"));
    }

    public static void setLocale(int locale){
        properties.setProperty("locale", String.valueOf(locale));
        Text.setLocale(locale);
    }

    public static boolean isDraggingEnabled(){
        return Boolean.valueOf(properties.getProperty("isDraggingEnabled"));
    }

    public static void setDraggingEnabled(Boolean dragging_enabled){
        properties.setProperty("isDraggingEnabled", dragging_enabled.toString());
    }

    public static boolean isMouseTrackingEnabled(){
        return Boolean.valueOf(properties.getProperty("isMouseTrackingEnabled"));
    }

    public static void setMouseTrackingEnabled(Boolean mouse_tracking_enabled){
        properties.setProperty("isMouseTrackingEnabled", mouse_tracking_enabled.toString());
    }


    /**
     * @return Application name
     */
    public static String getAppName(){
        return "PictoGame...";
    }

    /**
     * @return Application description
     */
    public static String getAppDescription(){
        return "A simple game for playing and creating nonograms";
    }

    /**
     * @return URL of this application
     */
    public static String getAppURL(){
        return "http://launchpad.net/pictogame";
    }

    /**
     * @return Version number
     */
    public static String getAppVersion(){
        return "0.1.1";
    }

    /**
     * @return Author
     */
    public static String getAppAuthor(){
        return "Federico Vera";
    }

    /**
     * @return Author's Email
     */
    public static String getAppAuthorEmail(){
        return "<dktcoding [at] gmail>";
    }

    /**DKT's application's path*/
    private final static String dktpath;
    /**Game path*/
    private final static String pictopath;
    /**Name of the file*/
    private static File file;

    private static boolean all_ready_checked = false;

    /**
     * Checks if the directory exists if it doesn't exist then it's created
     */
    private static void checkAndCreateDirectory(){
        if (all_ready_checked){
            return;
        }
        if (!file.exists()){
            if (!new File (dktpath).exists()){
                new File (dktpath).mkdir();
            }
            if (!new File (dktpath+pictopath).exists()){
                new File (dktpath+pictopath).mkdir();
                all_ready_checked = true;
            }
        }
    }

    /**
     * Saves the current configuration
     */
    public static void saveConfig(){
        try {
            FileOutputStream fos = new FileOutputStream(file);
            properties.storeToXML(fos, null);
        } catch (FileNotFoundException ex) {
            Logger.getLogger(Config.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(Config.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * Load the saved config
     * @return success
     */
    private static boolean loadConfig(){
        boolean state = false;
        try {
            properties = new Properties();
            file = new File(dktpath+pictopath+"/pictogame.conf");
            checkAndCreateDirectory();
            FileInputStream fis = new FileInputStream(file);
            properties.loadFromXML(fis);
            state = true;
        } catch (FileNotFoundException ex) {
            properties = (Properties)defProperties.clone();
        } catch (IOException ex) {
            Logger.getLogger(Config.class.getName()).log(Level.SEVERE, null, ex);
        }
        return state;
    }
    
    
    public void resetToDefaults(){
        properties = (Properties)defProperties.clone();
    }

    private static void loadDefaults(){
        defProperties = new Properties();
        defProperties.put("color.strikedTextForeground", "-65536");//Red
        defProperties.put("color.headerFontForeground", "-16777216");//Black
        defProperties.put("color.headerBorderColor", "-8355712");//Gray
        defProperties.put("color.headerBackgroundColor", "-4144960");//Light gray
        defProperties.put("color.headerBackgroundFocusColor", "-16711681");//Cyan
        defProperties.put("color.colorButtonBackgroundEmpty", "-1");//White
        defProperties.put("color.colorButtonBackgroundNothing", "-16711936");//Green
        defProperties.put("color.colorButtonBackgroundSomething", "-16777216");//Black
        defProperties.put("color.colorButtonBorderColor", "-8355712");//Gray
        defProperties.put("color.backgroundColor", "-6710785");
        defProperties.put("font.headerFont.name", "DejaVu Sans");
        defProperties.put("font.headerFont.style", "0");
        defProperties.put("font.headerFont.size", "9");
        defProperties.put("locale", "0");
        defProperties.put("isDraggingEnabled", "True");
        defProperties.put("isMouseTrackingEnabled", "True");
    }
}
