/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.utils;

import picto.app.interfaces.Localizable;
import picto.app.interfaces.Locales;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 * This class controls all of the localization events, I use this class for all
 * of my projects... so if something seems weird it probably is... <br>
 * The <code>WeakLocalizable</code> inner class doesn't make much sense for
 * <code>PictoGame</code> since all of the localizable objects are not
 * dereferenced during the game, but since I copy and paste this class from one
 * project to another just let it be!
 *
 * @see picto.app.interfaces.Localizable
 * @author Fido <dktcoding [at] gmail>
 */
public class Text implements Locales{
    private static ArrayList<WeakLocalizable> LOCALIZABLES
                                 = new ArrayList<WeakLocalizable>(20);
    private final static String STR_PATH = "/res/i18n/strings";
    private static Properties LOCAL_TEXT = new Properties();
    private static int currentLocale = -1;

    static{
        setLocale(Config.getLocale());
    }

    /**
     * Don't let anyone initialize this class
     */
    private Text(){

    }

    /**
     * Returns the current locale
     *
     * @return current locale
     */
    public static int getLocale(){
        return currentLocale;
    }

    /**
     * This method sets the locale of the application
     *
     * @param new_locale select one locale of the Locales list
     * @see picto.app.interfaces.Locales
     */
    public static void setLocale(int newLocale){
        if (currentLocale == newLocale){
            return;
        }
        LOCAL_TEXT.clear();
        JFileChooser.setDefaultLocale(locales[newLocale]);
        JOptionPane.setDefaultLocale(locales[newLocale]);
        try {
            String ID = "";
            if (newLocale != ENGLISH){
                ID = "_" + locales[newLocale].getLanguage();
            }
            ID += ".properties";
            LOCAL_TEXT.load(new InputStreamReader(Text.class
                                    .getResourceAsStream(STR_PATH + ID)));
        } catch (IOException ex) {
            Logger.getLogger(Text.class.getName()).log(Level.SEVERE, null, ex);
        }
        currentLocale = newLocale;
        update();
    }

    /**
     * Adds a new weak reference to a created object in order to update it.
     *
     * @param e Localizable object to be added to the list
     * @see picto.app.interfaces.Localizable
     */
    public static void addWeakReference(Localizable e){
        LOCALIZABLES.add(new WeakLocalizable(e));
    }

    /**
     * Updates all the localizable objects that were created
     */
    private static void update(){
        ArrayList<WeakLocalizable> temp =
                new ArrayList<WeakLocalizable>(LOCALIZABLES.size());
        for (int i = 0; i < LOCALIZABLES.size(); i++){
            if (LOCALIZABLES.get(i).get() != null){
                LOCALIZABLES.get(i).get().updateTexts();
                temp.add(LOCALIZABLES.get(i));
            } 
        }
        LOCALIZABLES.clear();
        LOCALIZABLES.addAll(temp);
    }

    /**
     * Method that returns the string to be displayed, if no string is found
     * then it falls back to English
     *
     * @param msg
     * @return Localized string
     */
    public synchronized static String _(String msg) {
        String text = LOCAL_TEXT.getProperty(msg);
        return (text == null) ? msg : text;
    }

    /**
     * This classes provides the weak references to the localizable objects
     */
    private static class WeakLocalizable extends WeakReference<Localizable>{
        private WeakLocalizable(Localizable referent){
            super(referent);
        }
    }
}
